--[[ EXAMPLE : LOGO

Description:
  A reflexion game.
  You must move the tiles by clicking on it
  and reassemble the Lua logo.
  
  This program may require the picture "LuaExamp/lualogo".
  
Illustrates:
  * "draw" package
  * picture manipulation
  * pen usage
]]

require "draw"
require "io"

function getTiles()

 local tiles
   -- look if the tiles already exist in a file

 local f = 
   io.file("LuaExamp","logotile")
 if f:exists() then
  f:open('r')
  tiles = f:read()
  f:close()
  return tiles
 end

  -- otherwhise, we have to create the tiles from the image

  -- load the original image
 local base = pict.load("LuaExamp","lualogo")
 tiles = {}
 local n=1

 -- cut it into 16 tiles
 for i=0,3 do
  for j=0,3 do
  tiles[n] = 
    pict.part(base, 14*j, 14*i, 14*j+13, 14*i+13)
   n=n+1
  end
 end

 -- now draw the 'empty' tile
 draw.onpict(tiles[16])
 draw.clear()
 i=0
 while i<14 do
   draw.line(0,i,i,0)
   draw.line(i+1,13,13,i+1)
   i=i+2
 end
 draw.onscreen()

 -- save the tiles for next time
 f:create("data", "Logo")
 f:open('w')
 f:write(tiles)
 f:close()

 return tiles

end

----------------------------

  -- compute a new initial state
function getInitState()
 local s = {{0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}}
 local p = {}
 local function rand()
  return math.floor(
    math.random()*16)+1
 end
 local v
 for i=1,4 do  for j=1,4 do
  repeat v = rand() 
  until p[v] == nil
  s[i][j] = v
  p[v] = 1
 end end

 return s
end

----------------------------

function drawGame(state, tiles)

 draw.rect(49,60,110,121,1,1,1)
 draw.rect(50,61,109,120,1,0)
 for i=0,3 do  for j=0,3 do
  draw.pict(14*j+52, 14*i+63, tiles[ state[i+1][j+1] ])
 end end
 draw.update()

end

----------------------------

function makeMove(state,x,y)
   -- move the tile
 local t = state[y][x]
 if y>1 and 
    state[y-1][x]==16 then
  state[y-1][x] = t
  t = 16
 elseif y<4 and 
    state[y+1][x]==16 then
  state[y+1][x] = t
  t = 16
 elseif x>1 and 
    state[y][x-1]==16 then
  state[y][x-1] = t
  t = 16
 elseif x<4 and 
    state[y][x+1]==16 then
  state[y][x+1] = t
  t = 16
 end
 state[y][x] = t
   
  -- check if the game is over
 local n=1
 for i=1,4 do  for j=1,4 do
  if state[i][j]~=n then
   return false
  end
  n=n+1
 end end

 return true
end

----------------------------

showgraph()
tiles = getTiles()
state = getInitState()
draw.onbuffer()

draw.text(24,20,"Move the tiles to make")
draw.text(29,35,"the Lua logo appear")

drawGame(state,tiles)
repeat
 x,y=waitpen()
 x,y = x-52,y-63
 if x>0 and y>0 then
  x,y = math.floor(x/14+1),
     math.floor(y/14+1)
  if x<5 and y<5 then
   goal = makeMove(state,x,y)
   drawGame(state,tiles)
  end
 end
until goal

draw.text(56,130,"Well done!")
draw.text(48,145,"Press a key...")
waitkey()

