--[[ EXAMPLE : MINES

Description:
  This is a classical Mines game.
  Open a case by clicking on it. 
  Place a flag by moving the pen to the right while
  holding it down on the screen.
  
Illustrates:
  * "draw" package
  * picture manipulation
  * pen usage
]]


-- REQUIRED LIBS AND PACKAGES

require("draw")


-- CONSTANTS AND FUNCTIONS

  -- you may freely change this 
GRID_WIDTH = 9
GRID_HEIGHT = 9
NBR_MINES = 10

----------------------------

  -- but don't change the following lines
HIDDEN = 0
FLAGGED = 1
BADFLAG = 2
OPEN = 3
BOOM = 4

MINE = -1

local D_COL = { 0, 1, 1, 1, 0,-1,-1,-1 }
local D_LG = { 1, 1, 0,-1,-1,-1, 0, 1 }
  
function rand(max)
  return 1 + math.floor(math.random()*max)
end

local PICTS = {}

function createPicts()
  local p
  local old = draw.onwhat()
  
    -- mine
  p = pict.new(11,11)
  draw.onpict(p)
  draw.rect(3,3,7,7,1,1,1)
  draw.line(2,2,8,8)
  draw.line(8,2,2,8)
  draw.line(1,5,9,5)
  draw.line(5,1,5,9)
  draw.pixel(4,4,0)
  PICTS.mine = p

    -- flag
  p = pict.new(11,11)
  draw.onpict(p)
  draw.line(3,9,7,9)
  draw.line(4,8,6,8)
  draw.line(5,1,5,7)
  draw.rect(6,1,7,3,1,1,1)
  draw.rect(7,2,8,4,1,1,1)
  PICTS.flag = p

    -- bad flag
  p = pict.part(PICTS.mine,0,0,10,10)
  draw.onpict(p)
  draw.rect(4,4,6,6,1,0,0)
  PICTS.badflag = p
  
     -- hidden
  p = pict.new(11,11)
  draw.onpict(p)
  draw.line(0,0,10,10)
  draw.line(0,2,8,10)
  draw.line(0,4,6,10)
  draw.line(0,6,4,10)
  draw.line(0,8,2,10)
  draw.pixel(0,10)
  draw.line(2,0,10,8)
  draw.line(4,0,10,6)
  draw.line(6,0,10,4)
  draw.line(8,0,10,2)
  draw.pixel(10,0)
  PICTS.hidden = p

  draw.onpict(old)
end 

----------------------------

function isInGrid(grid, lg, col)
  return (lg>0 and col>0 and lg<=grid.height and col<=grid.width)
end

----------------------------

function makeNewGrid(w, h, nbMines)

  local grid = {width=w, height=h, nbMines=nbMines}
  
  -- initialize the new grid
  
  grid.values = {}
  grid.states = {}
  
  for lg = 1,h do
    local val_lg = {}
    local states_lg = {}
    
    for col = 1,w do
      val_lg[col] = 0
      states_lg[col] = HIDDEN
    end
    
    grid.values[lg] = val_lg
    grid.states[lg] = states_lg
  end
    
  grid.nbOpened = 0
  
  -- place mines
  
  local placedMines = 0
  while placedMines < nbMines do
    local b_col = rand(w)
    local b_lg = rand(h)
    
    if grid.values[b_lg][b_col] == 0 then
      grid.values[b_lg][b_col] = MINE
      placedMines = placedMines + 1
    end
  end
  
  -- count the mines around each case
    
  for lg = 1,h do  for col = 1,w do
  
    if grid.values[lg][col] ~= MINE then
    
      local count = 0
      for i=1,8 do
        if isInGrid(grid, lg+D_LG[i], col+D_COL[i]) 
            and grid.values[ lg+D_LG[i] ][ col+D_COL[i] ] == MINE then
          count = count+1
        end 
      end
      
      grid.values[lg][col] = count
    end
    
  end end
  
  -- define position of the grid
  
  grid.x = math.floor((160 - w*(11+1) - 1)/2)
  grid.y =  math.floor((200 - h*(11+1) - 1)/2)
    
  return grid
end

----------------------------

function drawGrid(grid)
  
  local x,y = grid.x, grid.y
  local max_x = grid.x+grid.width*(11+1)
  local max_y = grid.y+grid.height*(11+1)
  
  for lg=1,grid.height+1 do
    draw.line(grid.x, y, max_x, y)
    y = y + (11+1)
  end
  
  for col=1,grid.width+1 do
    draw.line(x, grid.y, x, max_y)
    x = x + (11+1)
  end
  
  for lg = 1,grid.height do
    local val_lg = grid.values[lg]
    local states_lg = grid.states[lg]
    
    for col = 1,grid.width do
      local v = val_lg[col]
      local s = states_lg[col]
      x,y = grid.x + 1 + (col-1)*(11+1), grid.y + 1 + (lg-1)*(11+1)
      
      if s == HIDDEN then
        draw.pict(x,y,PICTS.hidden)
      elseif s == FLAGGED then 
        draw.pict(x,y,PICTS.flag)
      elseif s == BADFLAG then 
        draw.pict(x,y,PICTS.badflag)
      elseif s == BOOM then 
        draw.pict(x,y,PICTS.mine)
        draw.rect(x,y,x+10,y+10,1,2,2)
      else  
        if v == MINE then draw.pict(x,y,PICTS.mine)
        elseif v ~= 0 then draw.text(x+3,y+1,v)
        end
      end

    end
  end
  
end

----------------------------

function openZeroCase(grid, lg, col)
  if isInGrid(grid, lg, col) == false 
      or grid.states[lg][col] ~= HIDDEN then return end
  
  grid.states[lg][col] = OPEN
  grid.nbOpened = grid.nbOpened+1
    
  if grid.values[lg][col] == 0 then
    for i=1,8 do 
      openZeroCase(grid, lg+D_LG[i], col+D_COL[i])
    end
  end
end


function unhideMines(grid)
  for lg = 1,grid.height do
    local val_lg = grid.values[lg]
    local states_lg = grid.states[lg]
    
    for col = 1,grid.width do
      if val_lg[col] == MINE and states_lg[col] == HIDDEN then
        states_lg[col] = OPEN
      elseif val_lg[col] ~= MINE and states_lg[col] == FLAGGED then
        states_lg[col] = BADFLAG
      end
    end
  end
end


function flagAllMines(grid)
  for lg = 1,grid.height do
    local val_lg = grid.values[lg]
    local states_lg = grid.states[lg]
    
    for col = 1,grid.width do
      if val_lg[col] == MINE and states_lg[col] == HIDDEN then
        states_lg[col] = FLAGGED
      end
    end
  end
end


function openCase(grid, lg, col)

if grid.states[lg][col] ~= HIDDEN then return 0 end
  
  local v = grid.values[lg][col]
  if v == MINE then
    unhideMines(grid)
    grid.states[lg][col] = BOOM
    return -1
  else
    if v == 0 then 
      openZeroCase(grid, lg, col)
    else 
      grid.states[lg][col] = OPEN
      grid.nbOpened = grid.nbOpened+1
    end
    
    if grid.nbOpened == grid.width*grid.height - grid.nbMines then
     flagAllMines(grid)
     return 1
    else return 0
    end
  end
end

----------------------------

function setFlag(grid, lg, col)
  local st = grid.states[lg]
  if st[col] == HIDDEN then 
    st[col] = FLAGGED
  elseif st[col] == FLAGGED then
    st[col] = HIDDEN
  end
end

----------------------------

function point2coord(grid,x,y)
  return 1 + math.floor((y-grid.y)/(11+1)),
     1 + math.floor((x-grid.x)/(11+1))
end

function waitCommand(grid)
  local x,y = waitpen()
  local lg, col = point2coord(grid,x,y)
  
  local lg2,col2 = lg,col
  repeat
    x,y = getpen()
    if x ~= nil then
      lg2, col2 = point2coord(grid,x,y)
    end
  until x == nil
  
  if lg2==lg and col2>col then 
    return lg,col,true -- place a flag
  else
    return lg2,col2,false
  end
end

----------------------------
----------------------------

showgraph()
draw.onbuffer()

createPicts()
local grid = makeNewGrid(GRID_WIDTH,GRID_HEIGHT,NBR_MINES)
local lg, col, flag
local res = 0

repeat

  draw.clear()
  drawGrid(grid)
  draw.update()
  
  repeat 
    lg, col, flag = waitCommand(grid)
  until isInGrid(grid, lg, col)
  
  if(flag) then setFlag(grid, lg, col)
  else res = openCase(grid, lg, col)
  end
  
until res~=0
  
draw.clear()
drawGrid(grid)

if res==1 then 
  draw.text(52,10,"Nice job :)")
else
  draw.text(53,10,"Bad luck :(")
end
draw.update()

waitkey()

