--[[ EXAMPLE : PONG

Description:
  A classical Pong game.
  
Illustrates:
  * "draw" package
  *  keypad usage
  
Training:
  * Make the ball take some other angles than 45
  * Make the ball's speed progressively increase 
  * Make the calculator control the upper bar, and
    the user control the lower bar
]]

-- REQUIRED LIBS AND PACKAGES

require("draw")

-- CONSTANTS

TOP = 10
BOTTOM = 160
LEFT = 10
RIGHT = 150
BAR_UP = TOP+10
BAR_DN = BOTTOM-10
BAR_WTH = 18
R = 4

-- FUNCTIONS

function drawgame(bx,by,barx)
 draw.clear()
 draw.rect(LEFT,TOP,RIGHT,                BOTTOM)
 draw.circle(bx,by,R,1,1,1)
 draw.line(barx-BAR_WTH,BAR_UP,   barx+BAR_WTH,BAR_UP,1,2)
 draw.line(barx-BAR_WTH,BAR_DN,   barx+BAR_WTH,BAR_DN,1,2)    
 draw.update()
end

--------

function playgame()
 local x = 10 + 
   math.floor(math.random()*140)
 local y = 50 + 
   math.floor(math.random()*150)
 local dx,dy = 1,1
 local barx = (RIGHT+LEFT)/2
 local score = 0

 repeat

  x, y = x+dx, y+dy
  if x>=RIGHT-R or x<=LEFT+R       then dx=-dx end
  if y<=BAR_UP+R or y>=BAR_DN-R
    then 
    if x>=barx-BAR_WTH and             x<=barx+BAR_WTH then
    dy=-dy score=score+1 else
    lost = 1 end
 end

  if testkey(K_LEFT) and
    barx>LEFT+BAR_WTH then
   barx = barx-1
  elseif testkey(K_RIGHT) and
    barx<RIGHT-BAR_WTH then
   barx = barx+1
  end
 
  drawgame(x,y,barx)

 until lost or testkey(K_EXE)
 return score
end

-- MAIN PROGRAM

showgraph()
draw.onbuffer()
keypad(0)
sc = playgame()
print("Score: "..sc)

----------------------------
